#include <stdio.h>
#include <string.h>
#include <glib-object.h>

#include "exportable.h"
#include "uobject.h"
#include "music.h"
#include "upkg.h"

#define MUSIC_GET_PRIV(o) \
	G_TYPE_INSTANCE_GET_PRIVATE(o, ENGINE_MUSIC_TYPE, struct music_priv)

struct music_priv {
	struct upkg_file *f;
};

static void exportable_init(UObjectExportable *);

G_DEFINE_DYNAMIC_TYPE_EXTENDED(EngineMusic, engine_music, U_OBJECT_TYPE, 0,
	G_IMPLEMENT_INTERFACE(U_OBJECT_TYPE_EXPORTABLE, exportable_init)
);

static int export(GObject *o, FILE *f)
{
	struct music_priv *priv = MUSIC_GET_PRIV(o);
	unsigned char buf[1024];

	if (!priv->f || upkg_export_seek(priv->f, 0, SEEK_SET) != 0)
		return -1;

	while (!priv->f->eof) {
		size_t rc = upkg_export_read(priv->f, buf, sizeof buf);
		if (rc == 0) {
			if (priv->f->eof) break;
			return -1;
		}

		if (fwrite(buf, 1, rc, f) != rc)
			return -1;
	}

	return 0;
}

static int export_name(GObject *o, char *buf, size_t n)
{
	struct music_priv *priv = MUSIC_GET_PRIV(o);
	return snprintf(buf, n, "%s", priv->f ? priv->f->name : "");
}

static void exportable_init(UObjectExportable *e)
{
	e->export      = export;
	e->export_name = export_name;
}

static int deserialize(UObject *o, struct upkg_file *f)
{
	struct music_priv *priv = MUSIC_GET_PRIV(o);
	EngineMusic *m = ENGINE_MUSIC(o);
	size_t rc, pos = 0, buflen;
	unsigned char buf[32];
	long size;

	U_OBJECT_CLASS(engine_music_parent_class)->deserialize(o, f);

	buflen = upkg_export_read(f, buf, sizeof buf);

	/* Random field #1 */
	if (buflen - pos < 1)
		return -1;
	pos += 1;

	if (f->pkg->version > 61) {
		/* Random field #2 */
		if (buflen - pos < 4)
			return -1;
		pos += 4;
	}

	rc = upkg_decode_index(&size, buf+pos, buflen-pos);
	if (rc == 0 || size < 0)
		return -1;
	pos += rc;

	f->base += pos;
	f->len   = size;
	upkg_export_seek(f, 0, SEEK_SET);

	priv->f = f;

	return 0;
}

void music_register(GTypeModule *m)
{
	engine_music_register_type(m);
}

static void engine_music_init(EngineMusic *m)
{
}

static void engine_music_class_init(EngineMusicClass *class)
{
	UObjectClass *uo = U_OBJECT_CLASS(class);

	g_type_class_add_private(class, sizeof (struct music_priv));

	uo->deserialize = deserialize;
}

static void engine_music_class_finalize(EngineMusicClass *class)
{
}
